/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2008                                              */
/*                                                                           */
/* This software is licensed under the Eclipse Public License (EPL) V1.0.    */
/*****************************************************************************/

import com.zerog.ia.api.pub.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

public class STAFAdvancedWindowsOptions extends CustomCodePanel
                                        implements ActionListener
{
    private boolean inited = false;
    private JComboBox envMenuCB;
    private JCheckBox startSTAFCB;
    private JCheckBox startSTAFProcMinimizedCB;
    private JCheckBox createStartMenuIconsCB;
    private JComboBox ipvCB;
    private JComboBox perlCB;
    private JTextField instanceNameTF;
    private CustomCodePanelProxy ccpp;

    public boolean setupUI(CustomCodePanelProxy ccpp)
    {
        this.ccpp = ccpp;

        if (inited)
        {
            return true;
        }

        inited = true;

        String osname = ccpp.substitute("$prop.os.name$");
        String osarch = ccpp.substitute("$prop.os.arch$");

        GUIAccess gui =
            (GUIAccess)ccpp.getService(com.zerog.ia.api.pub.GUIAccess.class);
        Frame f = gui.getFrame();
        Font frameFont = f.getFont();
        Font boldFrameFont = new Font(frameFont.getName(), Font.BOLD,
                                      frameFont.getSize());

        JPanel outerPanel = new JPanel();
        outerPanel.setLayout(new BorderLayout());

        JPanel titlePanel = new JPanel();
        titlePanel.setBorder(BorderFactory.createLineBorder(Color.DARK_GRAY));
        titlePanel.setLayout(new BorderLayout());
        JLabel titleLabel = new
            JLabel(" Please choose the options that you want.",
            SwingConstants.LEFT);
        titleLabel.setFont(frameFont);
        titlePanel.add(BorderLayout.NORTH, titleLabel);
        titlePanel.setBackground(Color.white);
        titlePanel.setMinimumSize(new Dimension(300, 200));
        titlePanel.setPreferredSize(new Dimension(300, 50));
        titlePanel.setMaximumSize(new Dimension(500, 200));

        outerPanel.add(titlePanel, BorderLayout.NORTH);

        JPanel optionsPanel = new JPanel();
        GridBagLayout gbl = new GridBagLayout();
        GridBagConstraints gbc = new GridBagConstraints();
        optionsPanel.setLayout(gbl);
        optionsPanel.setBackground(Color.WHITE);

        JLabel envMenuTitle = new JLabel("Update Environment/Menus for:");
        envMenuTitle.setForeground(Color.black);
        envMenuTitle.setFont(boldFrameFont);

        gbc.anchor = GridBagConstraints.NORTHWEST;
        optionsPanel.add(envMenuTitle, gbc);
        optionsPanel.add(Box.createHorizontalStrut(10), gbc);

        envMenuCB = new JComboBox();
        envMenuCB.setFont(frameFont);
        envMenuCB.addItem("System");
        envMenuCB.addItem("User");
        envMenuCB.addItem("None");
        envMenuCB.setSelectedIndex(0);
        envMenuCB.addActionListener(this);

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.weightx = 1.0;
        optionsPanel.add(envMenuCB, gbc);

        gbc.weightx = 0;
        optionsPanel.add(Box.createVerticalStrut(5), gbc);

        startSTAFCB = new JCheckBox("Start STAF on user login",
            true);
        startSTAFCB.setFont(frameFont);
        startSTAFCB.setBackground(Color.WHITE);

        gbc.anchor = GridBagConstraints.NORTHWEST;
        optionsPanel.add(startSTAFCB, gbc);
        optionsPanel.add(Box.createHorizontalStrut(10), gbc);

        gbc.weightx = 0;
        optionsPanel.add(Box.createVerticalStrut(5), gbc);

        createStartMenuIconsCB = new
            JCheckBox("Create Start menu icons",
            true);
        createStartMenuIconsCB.setFont(frameFont);
        createStartMenuIconsCB.setBackground(Color.WHITE);

        gbc.anchor = GridBagConstraints.NORTHWEST;
        optionsPanel.add(createStartMenuIconsCB, gbc);
        optionsPanel.add(Box.createHorizontalStrut(10), gbc);

        gbc.weightx = 0;
        optionsPanel.add(Box.createVerticalStrut(5), gbc);

        startSTAFProcMinimizedCB = new
            JCheckBox("Start STAFProc minimized",
            false);
        startSTAFProcMinimizedCB.setFont(frameFont);
        startSTAFProcMinimizedCB.setBackground(Color.WHITE);

        gbc.anchor = GridBagConstraints.NORTHWEST;
        optionsPanel.add(startSTAFProcMinimizedCB, gbc);
        optionsPanel.add(Box.createHorizontalStrut(10), gbc);

        gbc.weightx = 0;
        optionsPanel.add(Box.createVerticalStrut(5), gbc);

        JLabel ipvLabel = new JLabel("Default TCP version:");
        ipvLabel.setForeground(Color.black);
        ipvLabel.setFont(boldFrameFont);

        gbc.anchor = GridBagConstraints.NORTHWEST;
        optionsPanel.add(ipvLabel, gbc);
        optionsPanel.add(Box.createHorizontalStrut(10), gbc);

        ipvCB = new JComboBox();
        ipvCB.setFont(frameFont);
        ipvCB.addItem("IPV4");
        ipvCB.addItem("IPV4_IPV6 (requires OS IPv6 support)");
        ipvCB.setSelectedIndex(0);
        ipvCB.addActionListener(this);

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.weightx = 1.0;
        optionsPanel.add(ipvCB, gbc);

        gbc.weightx = 0;
        optionsPanel.add(Box.createVerticalStrut(5), gbc);

        JLabel perlLabel = new JLabel("Default Perl version:");
        perlLabel.setForeground(Color.black);
        perlLabel.setFont(boldFrameFont);

        if (osarch.equals("x86") || osarch.equals("amd64"))
        {
            gbc.anchor = GridBagConstraints.NORTHWEST;
            optionsPanel.add(perlLabel, gbc);
            optionsPanel.add(Box.createHorizontalStrut(10), gbc);
        }

        perlCB = new JComboBox();
        perlCB.setFont(frameFont);
        perlCB.addItem("Perl 5.8");
        perlCB.addItem("Perl 5.10");

        if (osarch.equals("x86"))
        {
            perlCB.addItem("Perl 5.6");
        }

        perlCB.setSelectedIndex(0);
        perlCB.addActionListener(this);

        if (osarch.equals("x86") || osarch.equals("amd64"))
        {
            gbc.gridwidth = GridBagConstraints.REMAINDER;
            gbc.weightx = 1.0;
            optionsPanel.add(perlCB, gbc);

            gbc.weightx = 0;
            optionsPanel.add(Box.createVerticalStrut(5), gbc);
        }

        JLabel instanceNameLabel = new JLabel("Default STAF Instance Name:");
        instanceNameLabel.setForeground(Color.black);
        instanceNameLabel.setFont(boldFrameFont);

        gbc.anchor = GridBagConstraints.NORTHWEST;
        optionsPanel.add(instanceNameLabel, gbc);
        optionsPanel.add(Box.createHorizontalStrut(10), gbc);

        instanceNameTF = new JTextField("STAF", 30);

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.weightx = 1.0;
        optionsPanel.add(instanceNameTF, gbc);

        gbc.weightx = 0;
        optionsPanel.add(Box.createVerticalStrut(5), gbc);

        outerPanel.add(optionsPanel, BorderLayout.CENTER);

        add(outerPanel);

       return true;
    }

    public void panelIsDisplayed()
    {
    }

    public boolean okToContinue()
    {
        String envMenuSelection = (String)(envMenuCB.getSelectedItem());

        ccpp.setVariable("$UPDATE_ENVIRONMENT$", envMenuSelection);

        if (createStartMenuIconsCB.isSelected())
        {
            ccpp.setVariable("$CREATE_START_MENU_ICONS$", "1");
        }
        else
        {
            ccpp.setVariable("$CREATE_START_MENU_ICONS$", "0");
        }

        if (startSTAFCB.isSelected())
        {
            ccpp.setVariable("$START_ON_LOGIN$", "1");
        }
        else
        {
            ccpp.setVariable("$START_ON_LOGIN$", "0");
        }

        if (startSTAFProcMinimizedCB.isSelected())
        {
            ccpp.setVariable("$START_STAFPROC$", "Minimized");
        }
        else
        {
            ccpp.setVariable("$START_STAFPROC$", "Normal");
        }

        String ipvSelection = (String)(ipvCB.getSelectedItem());

        if (ipvSelection.equals("IPV4"))
        {
            ccpp.setVariable("$USE_TCP_VERSION$", "IPV4");
        }
        else if (ipvSelection.equals("IPV4_IPV6 (requires OS IPv6 support)"))
        {
            ccpp.setVariable("$USE_TCP_VERSION$", "IPV4_IPV6");
        }

        String perlSelection = (String)(perlCB.getSelectedItem());

        if (perlSelection.equals("Perl 5.8"))
        {
            ccpp.setVariable("$USE_PERL_VERSION$", "5.8");
        }
        else if (perlSelection.equals("Perl 5.6"))
        {
            ccpp.setVariable("$USE_PERL_VERSION$", "5.6");
        }
        else if (perlSelection.equals("Perl 5.10"))
        {
            ccpp.setVariable("$USE_PERL_VERSION$", "5.10");
        }

        ccpp.setVariable("$STAF_INSTANCE_NAME$", instanceNameTF.getText());

        return true;
    }

    public boolean okToGoPrevious()
    {
        return true;
    }

    public String getTitle()
    {
        return "Advanced Options";
    }

    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource() == envMenuCB)
        {
            String envMenuSelection = (String)(envMenuCB.getSelectedItem());

            if (envMenuSelection.equals("None"))
            {
                createStartMenuIconsCB.setSelected(false);
                createStartMenuIconsCB.setEnabled(false);
                startSTAFCB.setSelected(false);
                startSTAFCB.setEnabled(false);
                startSTAFProcMinimizedCB.setEnabled(false);
            }
            else
            {
                createStartMenuIconsCB.setEnabled(true);
                startSTAFCB.setEnabled(true);
            }
        }
    }
}