/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2008                                              */
/*                                                                           */
/* This software is licensed under the Eclipse Public License (EPL) V1.0.    */
/*****************************************************************************/

import com.zerog.ia.api.pub.*;
import java.io.*;
import java.util.*;

public class STAFConsoleAdvancedOptions extends CustomCodeConsoleAction
{
    public boolean setup()
    {
        return true;
    }

    public void executeConsoleAction() throws PreviousRequestException
    {
        ConsoleUtils cu = (ConsoleUtils)cccp.getService(ConsoleUtils.class);

        String osname = cccp.substitute("$prop.os.name$");
        String osarch = cccp.substitute("$prop.os.arch$");

        Vector updateEnvironmentOptions = new Vector();
        updateEnvironmentOptions.add("System");
        updateEnvironmentOptions.add("User");
        updateEnvironmentOptions.add("None");

        int updateEnvironmentSelection =
            cu.createChoiceListAndGetValue("Update Environment/Menus for",
                                           updateEnvironmentOptions,
                                           0);

        String updateEnvironmentSelectionText = (String)
            updateEnvironmentOptions.elementAt(updateEnvironmentSelection);

        cccp.setVariable("$UPDATE_ENVIRONMENT$",
            updateEnvironmentSelectionText);

        cu.wprintln("\n");

        if ((osname.indexOf("Windows") > -1) &&
            !(updateEnvironmentSelectionText.equals("None")))
        {
            String startSTAF = cu.promptAndGetValueWithDefaultValue(
                "Start STAF on user login?", "1");

            cccp.setVariable("$START_ON_LOGIN$", startSTAF);

            cu.wprintln("\n");

            String createStartMenuIcons = cu.promptAndGetValueWithDefaultValue(
                "Create Start menu icons?", "1");

            cccp.setVariable("$CREATE_START_MENU_ICONS$", createStartMenuIcons);

            cu.wprintln("\n");

            Vector startSTAFProcOptions = new Vector();
            startSTAFProcOptions.add("Normal");
            startSTAFProcOptions.add("Minimized");

            int startSTAFProcSelection =
                cu.createChoiceListAndGetValue("Start STAFProc",
                                               startSTAFProcOptions,
                                               0);

            String startSTAFProcSelectionText = (String)
                startSTAFProcOptions.elementAt(startSTAFProcSelection);

            cccp.setVariable("$START_STAFPROC$", startSTAFProcSelectionText);

            cu.wprintln("\n");
        }

        Vector tcpOptions = new Vector();
        tcpOptions.add("IPV4");
        tcpOptions.add("IPV4_IPV6");

        int tcpSelection =
            cu.createChoiceListAndGetValue("Default TCP version",
                                           tcpOptions,
                                           0);

        cccp.setVariable("$USE_TCP_VERSION$",
            tcpOptions.elementAt(tcpSelection));

        cu.wprintln("\n");

        if ((osname.indexOf("Windows") > -1) && (osarch.equals("x86")) ||
            (osname.indexOf("Linux") > -1) && (osarch.equals("x86")) ||
            (osname.indexOf("Linux") > -1) && (osarch.equals("amd64")) ||
            (osname.indexOf("Linux") > -1) && (osarch.equals("ia64")) ||
            ((osname.indexOf("SunOS") > -1) && (osarch.equals("sparc"))) ||
            (osname.indexOf("AIX") > -1) ||
            (osname.indexOf("Mac OS X") > -1))
        {
            Vector perlOptions = new Vector();
            perlOptions.add("5.8");

            if (!((osname.indexOf("Linux") > -1) && (osarch.equals("ia64"))))
            {
                perlOptions.add("5.10");
            }

            if ((osname.indexOf("Linux") > -1) && (osarch.equals("x86")))
            {
                perlOptions.add("5.6");
            }

            int perlSelection =
                cu.createChoiceListAndGetValue("Default Perl version",
                                               perlOptions,
                                               0);

            cccp.setVariable("$USE_PERL_VERSION$",
                perlOptions.elementAt(perlSelection));

            cu.wprintln("\n");
        }

        String stafInstanceName = cu.promptAndGetValueWithDefaultValue(
            "Default STAF Intance Name", "STAF");

        cccp.setVariable("$STAF_INSTANCE_NAME$", stafInstanceName);

        cu.wprintln("\n");
    }

    public String getTitle()
    {
        return "Advanced Options";
    }
}