/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2008                                              */
/*                                                                           */
/* This software is licensed under the Eclipse Public License (EPL) V1.0.    */
/*****************************************************************************/

import com.zerog.ia.api.pub.*;
import java.io.*;

/*
  This CustomCodeAction is a workaround for an InstallAnywhere problem where
  on an upgrade of STAF, when the existing version of STAF is uninstalled
  during the upgrade, the Uninstaller for STAF will sometimes not be created.

  The problem is that when the uninstaller is launched, the process that is
  spawned will recursively check the uninstall folder and remove anything in
  it.  This process seems to linger around for a bit when the install sequence
  starts.  So when the new uninstaller is created, that process will remove it.

  The workaround is to have a CustomCodeAction delay for 5 seconds in the
  install sequence, after the uninstall.

  This problem was reported to InstallAnywhere support as incident
  #SIOA-000116325, and the work order for the fix is #IOA-000031200.
*/

public class STAFDelay extends CustomCodeAction
{

    public void install(InstallerProxy ip)
    {
        try
        {
            Thread.sleep(5000);
        }
        catch (InterruptedException ex)
        {
        }
    }

    public void uninstall(UninstallerProxy up)
    {
    }

    public String getInstallStatusMessage()
    {
        return "";
    }

    public String getUninstallStatusMessage()
    {
        return "";
    }
}