/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2004, 2005                                        */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

import com.installshield.wizard.*;
import com.installshield.wizard.service.*;
import com.installshield.util.*;
import com.installshield.product.service.product.*;
import java.io.*;
import java.util.*;

public class STAFRemoveUnneededCodepages extends WizardAction
{
    public void execute(WizardBeanEvent event)
    {
        String fileSep = System.getProperty("file.separator");
        String codepageDir = resolveString("$P(absoluteInstallLocation)") +
            fileSep + "codepage";
            
        String acp = resolveString("$W(stafCodepages.acp)");            
        String oemcp = resolveString("$W(stafCodepages.oemcp)");
        
        String acpFile = codepageDir + fileSep + "ibm-" + acp + ".bin";
        String oemcpFile = codepageDir + fileSep + "ibm-" + oemcp + ".bin";
        
        Vector requiredCodepages = new Vector();
        requiredCodepages.add(acpFile);
        requiredCodepages.add(oemcpFile);
        requiredCodepages.add(codepageDir + fileSep + "alias.txt");
            
        File codepageDirFile = new File(codepageDir);
        File [] codepageFiles = codepageDirFile.listFiles();
        
        for (int i = 0; i < codepageFiles.length; i++)
        {
            String codePageFileName = codepageFiles[i].toString();
            
            if (!requiredCodepages.contains(codePageFileName))
            {
                codepageFiles[i].delete();
            }
        }
    }    
    
    public void copy(InputStream in, OutputStream out)
    {
        synchronized(in)
        {
            synchronized(out)
            {
                try 
                {
                    byte [] buffer = new byte[256];
                    while (true)
                    {
                        int bytesRead = in.read(buffer);
                        
                        if (bytesRead == -1)
                        {
                            break;
                        }
                        
                        out.write(buffer, 0, bytesRead);
                    }
                }
                catch (IOException ex)
                {
                    ex.printStackTrace();
                }
            }
        }
    }
}