/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;

public class STAXSignalHandlerAction implements STAXAction
{
    private STAXSignalHandlerAction()
    { /* Do Nothing */ }

    public STAXSignalHandlerAction(String name, STAXAction action)
    {
        fUnevalName = name;
        fName = name;
        fAction = action;
    }

    public String getSignalName() { return fName; }

    public String getXMLInfo()
    {
        return "<signalhandler name=\"" + fName + "\"/>";
    }

    public String getInfo()
    {
        return fName;
    }

    public String getDetails()
    {
        return "Name:" + fName + 
               ";Action:" + fAction;
    }

    public void execute(STAXThread thread)
    {
        try
        {
            fName = thread.pyStringEval(fUnevalName);
        }
        catch (STAXPythonEvaluationException e)
        {
            thread.popAction();

            thread.pySetVar("STAXPythonEvalMsg", 
                            "\n" + getXMLInfo() + "\n\n" + e);
            thread.raiseSignal("STAXPythonEvaluationError");
            return;
        }

        thread.registerSignalHandler(fName,
            new STAXSignalExecutionAction(fName, fAction));
        thread.popAction();
    }

    public void handleCondition(STAXThread thread, STAXCondition cond)
    {
        thread.popAction();
    }

    public STAXAction cloneAction()
    {
        STAXSignalHandlerAction clone = new STAXSignalHandlerAction();

        clone.fUnevalName = fUnevalName;
        clone.fName = fName;
        clone.fAction = fAction;

        return clone;
    }

    private String fUnevalName = new String();
    private String fName = new String();
    private STAXAction fAction =  null;
}
