/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;

public class STAXTestcaseStatusActionFactory implements STAXActionFactory
{
    static final String STAX_TESTCASE_STATUS = new String("TestcaseStatus");

    private static String fDTDInfo =
"\n" +
"<!--================= The Testcase Status Element ================== -->\n" +
"<!--\n" +
"     Marks status result ('pass' or 'fail') for a testcase and\n" +
"     allows additional information to be specified.  The status\n" +
"     result and the additional info is evaluated via Python.\n" +
"-->\n" +
"<!ELEMENT tcstatus   (#PCDATA)>\n" +
"<!ATTLIST tcstatus\n" +
"          result     CDATA  #REQUIRED\n" +
">\n";

    public String getDTDInfo()
    {
        return fDTDInfo;
    }

    public String getDTDTaskName()
    {
        return "tcstatus";
    }

    public STAXAction parseAction(STAX staxService, STAXJob job,
                                  org.w3c.dom.Node root) throws STAXException
    {
        String status = null;
        String message = null;

        NamedNodeMap attrs = root.getAttributes();

        for (int i = 0; i < attrs.getLength(); ++i)
        {
            Node thisAttr = attrs.item(i);

            String errorInfo = "\n  Element: " + root.getNodeName() +
                               "  Attribute: " + thisAttr.getNodeName();

            if (thisAttr.getNodeName().equals("result"))
            {                
                status = STAXUtil.parseAndCompileForPython(
                    thisAttr.getNodeValue(), errorInfo);
            }
            else
            {
                throw new STAXInvalidXMLAttributeException(
                          root.getNodeName() + ": " + thisAttr.getNodeName());
            }
        }

        NodeList children = root.getChildNodes();

        for (int i = 0; i < children.getLength(); ++i)
        {
            Node thisChild = children.item(i);

            if (thisChild.getNodeType() == Node.COMMENT_NODE)
            {
                /* Do nothing */
            }
            else if (thisChild.getNodeType() == Node.TEXT_NODE)
            {
                String errorInfo = "\n  Element: " + root.getNodeName();

                message = STAXUtil.parseAndCompileForPython(
                    thisChild.getNodeValue(), errorInfo);
            }
            else
            {
                throw new STAXInvalidXMLNodeTypeException(
                          Integer.toString(thisChild.getNodeType()));
            }
        }

        return new STAXTestcaseStatusAction(status, message);
    }
}
