/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;

public class STAXThrowActionFactory implements STAXActionFactory
{
    private static String fDTDInfo =
"\n" +
"<!--================= The Throw Element ============================ -->\n" +
"<!--\n" +
"     The throw element specifies an exception to throw.\n" +
"     The exception attribute value and any additional information\n" +
"     is evaluated via Python.\n" +
"-->\n" +
"<!ELEMENT throw      (#PCDATA)>\n" +
"<!ATTLIST throw\n" +
"          exception  CDATA        #REQUIRED\n" +
">\n";

    public String getDTDInfo()
    {
        return fDTDInfo;
    }

    public String getDTDTaskName()
    {
        return "throw";
    }

    public STAXAction parseAction(STAX staxService, STAXJob job,
                                  org.w3c.dom.Node root) throws STAXException
    {
        String exName = null;
        String data = new String("None");
        NamedNodeMap attrs = root.getAttributes();

        for (int i = 0; i < attrs.getLength(); ++i)
        {
            Node thisAttr = attrs.item(i);

            String errorInfo = "\n  Element: " + root.getNodeName() +
                               "  Attribute: " + thisAttr.getNodeName();

            if (thisAttr.getNodeName().equals("exception"))
            {
                exName = STAXUtil.parseAndCompileForPython(
                    thisAttr.getNodeValue(), errorInfo);
            }
            else
            {
                throw new STAXInvalidXMLAttributeException(
                          root.getNodeName() + ": " + thisAttr.getNodeName());
            }
        }

        NodeList children = root.getChildNodes();

        for (int i = 0; i < children.getLength(); ++i)
        {
            Node thisChild = children.item(i);

            if (thisChild.getNodeType() == Node.COMMENT_NODE)
            {
                /* Do nothing */
            }
            else if (thisChild.getNodeType() == Node.TEXT_NODE)
            {
                String errorInfo = "\n  Element: " + root.getNodeName();

                data = STAXUtil.parseAndCompileForPython(
                    thisChild.getNodeValue(), errorInfo);
            }
            else
            {
                throw new STAXInvalidXMLNodeTypeException(
                          Integer.toString(thisChild.getNodeType()));
            }
        }

        return new STAXThrowAction(exName, data);
    }
}
