/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2001                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

#include "STAF.h"
#include "STAFFileSystem.h"

int main(void)
{
#ifdef STAF_OS_NAME_WIN32
    STAFString testPaths[] =
    {
        "c:\\windows",
        "c:\\staf\\bin\\STAF.cfg",
        "STAF.cfg",
        "c:",
        "",
        "windows/system32",
        "staf/bin/STAF.cfg",
        "\\windows\\",
        "a.b.c.d",
        "c:\\a.b\\c.d",
        "c:\\windows\\.a\\.b",
        "c:\\\\",
        "c:\\\\staf\\\\data\\\\log",
        "d:/stuff",
        "d:/stuff\\it/good.bad",
        "/stuff\\it",
        "d:\\c:\\",
        ":/stuff"
    };
#else
    STAFString testPaths[] =
    {
        "/usr/bin",
        "/usr/local/staf/STAF.cfg",
        "STAF.cfg",
        "/",
        "",
        "usr/bin",
        "usr/local/staf/STAF.cfg",
        "/usr/",
        "a.b.c.d",
        "/a.b/c.d",
        "/usr/.a/.b",
        "//",
        "//usr///bin//sh"
    };
#endif

    for (int i = 0; i < (sizeof(testPaths) / sizeof(STAFString)); ++i)
    {
        STAFFSPath path(testPaths[i]);

        cout << endl << endl << "Path: " << path.asString() << endl
             << "Root  : " << path.root() << endl
             << "# Dirs: " << path.numDirs() << endl;

        for (unsigned int j = 0; j < path.numDirs(); ++j)
            cout << "Dir #" << j << ": " << path.dir(j) << endl;

        cout << "Name  : " << path.name() << endl
             << "Ext   : " << path.extension() << endl;

        // Set the root so that the path gets recalculated

        path.setRoot(path.root());

        cout << "Path2 : " << path.asString() << endl;

        if (testPaths[i] != path.asString())
            cout << "Warning: Paths do not match" << endl;
    }

    return 0;
}
